// Включаем библиотеку для работы с портами ввода-вывода микроконтроллера AVR.
#include <avr/io.h>
// Определяем частоту процессора микроконтроллера в 1 МГц.
#define F_CPU 1000000UL
// Определяем бит в регистре PIND для сигнала с клока энкодера.
#define PIND_CLK 0b00010000
// Определяем бит в регистре PIND для сигнала с DT энкодера.
#define PIND_DT 0b01000000

volatile uint8_t next_state, prev_state, state;
int sw34, pin_clk_2, pin_dt_2;


void read_encoder_state()
{
    // Устанавливаем начальные значения переменных равное 0
    pin_clk_2 = 0;
    pin_dt_2 = 0;

    // Устанавливаем значение PIN_CLK_2 в зависимости от установки бита PIND_CLK в PIND
    pin_clk_2 = (PIND & PIND_CLK) ? 1 : 0;
    // Устанавливаем значение PIN_DT_2 в зависимости от установки бита PIND_DT в PIND
    pin_dt_2 = (PIND & PIND_DT) ? 1 : 0;
     // Определяем и сохраняем следующее состояние энкодера
    next_state = (pin_clk_2 << 1) | pin_dt_2;

    // Проверяем, изменилось ли состояние энкодера
    if (next_state != prev_state)
    {
        switch (next_state)
        {
            case 0: // CW
                if (prev_state == 3) state++;
                else if (prev_state == 0) state--;
                break;
            case 1: // CCW
                if (prev_state == 1) state++;
                else if (prev_state == 2) state--;
                break;
            case 2: // CW
                if (prev_state == 1) state++;
                else if (prev_state == 3) state--;
                break;
            case 3: // CCW
                if (prev_state == 2) state++;
                else if (prev_state == 0) state--;
                break;
        }
        // Устанавливаем значение prev_state равным текущему состоянию энкодера
        prev_state = next_state;
    }
}

void encoder_setup()
{
    // Инициализация последовательного порта
    Serial.begin(9600);

    // Настройка портов ввода-вывода
    DDRD = 0x00;
    PORTD = (PIND_CLK + PIND_DT);
}

uint8_t encoder_value()
{
    // Читаем состояние энкодера
    read_encoder_state();

    // Если состояние энкодера изменилось, выводим его в последовательный порт
    if (state != sw34)
    {
        return state;
        sw34 = state;
    }
}